#ifndef __BROADCAST_UPLOADER_HPP__
#define __BROADCAST_UPLOADER_HPP__

#include <iostream>
#include <exception>
#include <sstream>
#include <string>
#include <iterator> 
#include <algorithm>
#include <map>
#include <set>

#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/mman.h>

#include <boost/filesystem.hpp>
#include <openssl/md5.h>

#include <fstream>
#include <cstdlib>
#include <cerrno>
#include <curlpp/cURLpp.hpp>
#include <curlpp/Easy.hpp>
#include <curlpp/Options.hpp>
#include <curlpp/Exception.hpp>


#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>
#include <boost/foreach.hpp>


class BroadcastUploader {
    public:
    BroadcastUploader (std::string pLoginToken): mLoginToken(pLoginToken) {
        
    }

    ~BroadcastUploader () {
        
    }

    bool  login();
    bool  getSupportedFileTypes();
    int  getTargetFiles();
    bool confirm();
    int  uploadFiles();
    
    private:
        int  getMD5();
        bool parseLoginResponse(boost::property_tree::ptree const& pNode);
        bool parseSupportedFileTypesResponse(boost::property_tree::ptree const& pNode);
        int  parseMD5Response(boost::property_tree::ptree const& pNode);
        const std::string computeMD5(std::string path);
        int  uploadFile(std::string pFilePath);
        bool parseUploadResult(boost::property_tree::ptree const& pNode);
        
        std::string mLoginToken;
        
        std::map<std::string, std::string> mFieldValueMap;
        std::set<std::string> mSupportedExtensions;
        std::set<std::string> mTargetFilesList;
        std::set<std::string> mMD5List;
        
};

#endif


