#include <broadcastUploader.hpp>


#define QUOTE_STR(...) #__VA_ARGS__



using boost::property_tree::ptree;

int main(int argc, char *argv[]) {

    if (argc != 2) {
        std::cout<<"Run this script in the parent directory of your music files.\n";
        std::cout<<"To acquire a login token, enable the \"Simple Uploaders\" app by visiting https://ibroadcast.com, logging in to your account, and clicking the \"Apps\" button in the side menu.\n";
        std::cout<<"Syntax: ./ibroadcast-uploader <login_token>\n";
        exit(-1);
    }

    BroadcastUploader broadcastUploader (argv[1]);

    if (broadcastUploader.login()) {
    
        if (broadcastUploader.getSupportedFileTypes()) {
        
            broadcastUploader.getTargetFiles();

            if (broadcastUploader.confirm()) {
                broadcastUploader.uploadFiles();
            }
        
        }
        else {
            std::cout<<"Unable to get supported file types"<<std::endl;
        }
    }
    else {
        std::cout<<"Authorization failed"<<std::endl;
    }

    return 0;
}


bool BroadcastUploader::login()
{
    std::string loginStr = std::string("{") +
                                std::string(QUOTE_STR("mode":"login_token",)) +
                                std::string(QUOTE_STR("login_token":)) + std::string("\"") + mLoginToken + std::string("\",")  +
                                std::string(QUOTE_STR("version":1.1,)) +
                                std::string(QUOTE_STR("client":"c++ Linux uploader",)) +
                                std::string(QUOTE_STR("type":"account",)) +
                                std::string(QUOTE_STR("app_id":1007)) +
                           std::string("}");

    try 
    {
        boost::property_tree::ptree  root;
        curlpp::Cleanup cleaner;
        curlpp::Easy request;
        std::stringstream responseStream;

        request.setOpt(new curlpp::options::Url("https://api.ibroadcast.com/s/JSON/"));
        request.setOpt(new curlpp::options::Verbose(false)); 

        std::list<std::string> header;
        header.push_back("User-Agent: c++ Linux uploader");
        header.push_back("Content-Type: application/json"); 

        request.setOpt(new curlpp::options::HttpHeader(header)); 
        request.setOpt(new curlpp::options::PostFields(loginStr));
        request.setOpt(new curlpp::options::PostFieldSize(loginStr.length()));
        request.setOpt(new curlpp::options::WriteStream( &responseStream ) );
        
        request.perform(); 
        
        //std::cout<<"\nLogin resp:"<<responseStream.str()<<std::endl;
        boost::property_tree::read_json(responseStream, root);
   
        return parseLoginResponse(root);

//        std::copy(mSupportedExtensions.begin(),
//                  mSupportedExtensions.end(),
//                  std::ostream_iterator<std::string>(std::cout, " "));
    }
    catch ( curlpp::LogicError & e ) {
        std::cout << e.what() << std::endl;
    }
    catch ( curlpp::RuntimeError & e ) {
        std::cout << e.what() << std::endl;
    }
    
    return false;
}

bool BroadcastUploader::getSupportedFileTypes()
{
    std::string loginStr = std::string("{") +
                                std::string(QUOTE_STR("mode":"status",)) +
                                std::string(QUOTE_STR("user_id":)) + std::string("\"") + mFieldValueMap["id"] + std::string("\",")  +
                                std::string(QUOTE_STR("token":)) + std::string("\"") + mFieldValueMap["token"] + std::string("\",")  +
                                std::string(QUOTE_STR("version":1.1,)) +
                                std::string(QUOTE_STR("supported_types":1,)) +
                                std::string(QUOTE_STR("client":"c++ Linux uploader")) +
                           std::string("}");

    try
    {
        boost::property_tree::ptree  root;
        curlpp::Cleanup cleaner;
        curlpp::Easy request;
        std::stringstream responseStream;

        request.setOpt(new curlpp::options::Url("https://api.ibroadcast.com/s/JSON/"));
        request.setOpt(new curlpp::options::Verbose(false));

        std::list<std::string> header;
        header.push_back("User-Agent: c++ Linux uploader");
        header.push_back("Content-Type: application/json");

        request.setOpt(new curlpp::options::HttpHeader(header));
        request.setOpt(new curlpp::options::PostFields(loginStr));
        request.setOpt(new curlpp::options::PostFieldSize(loginStr.length()));
        request.setOpt(new curlpp::options::WriteStream( &responseStream ) );
        
        request.perform();
        
        //std::cout<<"\nLogin resp:"<<responseStream.str()<<std::endl;
        boost::property_tree::read_json(responseStream, root);
   
        return parseSupportedFileTypesResponse(root);

//        std::copy(mSupportedExtensions.begin(),
//                  mSupportedExtensions.end(),
//                  std::ostream_iterator<std::string>(std::cout, " "));
    }
    catch ( curlpp::LogicError & e ) {
        std::cout << e.what() << std::endl;
    }
    catch ( curlpp::RuntimeError & e ) {
        std::cout << e.what() << std::endl;
    }
    
    return false;
}



int BroadcastUploader::getMD5()
{

    boost::property_tree::ptree pt;
    
    if (mFieldValueMap["id"].empty() ||
        mFieldValueMap["token"].empty())
    {
        std::cout<<"missing id or token"<<std::endl;
        return -1;
    }
    
    std::string syncStr = "user_id="+ mFieldValueMap["id"] + "&token=" +  mFieldValueMap["token"];

    try 
    {
        boost::property_tree::ptree  root;
        curlpp::Cleanup cleaner;
        curlpp::Easy request;
        std::stringstream responseStream;

        request.setOpt(new curlpp::options::Url("https://upload.ibroadcast.com"));
        request.setOpt(new curlpp::options::Verbose(false)); 

        std::list<std::string> header; 
        header.push_back("Content-Type: application/x-www-form-urlencoded"); 

        request.setOpt(new curlpp::options::HttpHeader(header)); 
        request.setOpt(new curlpp::options::PostFields(syncStr));
        request.setOpt(new curlpp::options::PostFieldSize(syncStr.length()));
        request.setOpt(new curlpp::options::WriteStream( &responseStream ) );
        
        request.perform(); 
        
        //std::cout<<"\nResp:"<<responseStream.str()<<std::endl;
        boost::property_tree::read_json(responseStream, root);
   
        parseMD5Response(root);
    }
    catch ( curlpp::LogicError & e ) {
        std::cout << e.what() << std::endl;
    }
    catch ( curlpp::RuntimeError & e ) {
        std::cout << e.what() << std::endl;
    }
    
    return 0;
}

int BroadcastUploader::uploadFiles()
{
    getMD5();
    
    // upload a resource unless already present
    
    std::set<std::string>::iterator fileIter;
    for (fileIter = mTargetFilesList.begin();
         fileIter != mTargetFilesList.end();
         fileIter++) {
        std::string md5Value = computeMD5(*fileIter);
        if (mMD5List.find(md5Value) == mMD5List.end()) {
            std::cout<<"Uploading "<< *fileIter << "...";
            std::cout.flush();
            uploadFile(*fileIter);
        }
        else {
            std::cout<<"Skipping already existing resource: "<< *fileIter <<std::endl;
        }
    }

    return 0;
}

int BroadcastUploader::uploadFile(std::string pFilePath)
{
    try 
    {
        boost::property_tree::ptree  root;
        curlpp::Cleanup cleaner;
        curlpp::Easy request;
        std::stringstream responseStream;

        request.setOpt(new curlpp::options::Url("https://upload.ibroadcast.com"));
        request.setOpt(new curlpp::options::Verbose(false)); 
        
        {
          curlpp::Forms formParts;
          formParts.push_back(new curlpp::FormParts::Content("file", pFilePath.c_str()));
          formParts.push_back(new curlpp::FormParts::Content("file_path", pFilePath.c_str()));
          formParts.push_back(new curlpp::FormParts::Content("method", "C++ linux uploader"));
          formParts.push_back(new curlpp::FormParts::File("file", pFilePath.c_str()));

          formParts.push_back(new curlpp::FormParts::Content("user_id", mFieldValueMap["id"]));
          formParts.push_back(new curlpp::FormParts::Content("token", mFieldValueMap["token"]));
          request.setOpt(new curlpp::options::WriteStream(&responseStream));
          request.setOpt(new curlpp::options::HttpPost(formParts)); 
        }

        request.perform();
        
        //std::cout<<"\nResp: "<<responseStream.str()<<std::endl;
        boost::property_tree::read_json(responseStream, root);
   
        const char * resultStr= (parseUploadResult(root)) ? "OK" : "FAILED";
        std::cout<<resultStr<<std::endl;
                  
    }
    catch ( curlpp::LogicError & e ) {
        std::cout << e.what() << std::endl;
    }
    catch ( curlpp::RuntimeError & e ) {
        std::cout << e.what() << std::endl;
    }
          
    
    return 0;
}

bool BroadcastUploader::parseLoginResponse(boost::property_tree::ptree const& pNode)
{
    using boost::property_tree::ptree;    
    
    std::string strResult = pNode.get<std::string>("result");
    
    if (strResult != "true" ) {
        return false;
    }
    
    std::string id = pNode.get<std::string>("user.id");    
    mFieldValueMap["id"] = id;
    
    std::string token = pNode.get<std::string>("user.token");    
    mFieldValueMap["token"] = token;

    return true;
}

bool BroadcastUploader::parseSupportedFileTypesResponse(boost::property_tree::ptree const& pNode)
{
    using boost::property_tree::ptree;
    
    std::string strResult = pNode.get<std::string>("result");
    
    if (strResult != "true" ) {
        return false;
    }
    
    BOOST_FOREACH(const ptree::value_type& value,
                  pNode.get_child("supported")) {
        mSupportedExtensions.insert(value.second.get<std::string>("extension"));
    }

    return true;
}



int BroadcastUploader::parseMD5Response(boost::property_tree::ptree const& pNode)
{
    using boost::property_tree::ptree;
    
    BOOST_FOREACH(const ptree::value_type& value,
                  pNode.get_child("md5")) {
        mMD5List.insert(value.second.get_value<std::string>());
    }

    return 0;
}

bool BroadcastUploader::parseUploadResult(boost::property_tree::ptree const& pNode)
{
    using boost::property_tree::ptree;
  
    std::string strResult = pNode.get<std::string>("result");
    
    return (strResult == "true");
}

int BroadcastUploader::getTargetFiles()
{
    using namespace boost::filesystem;

    path currentPath = boost::filesystem::current_path();
    recursive_directory_iterator itr(currentPath);
    while (itr != recursive_directory_iterator()) {
        
        if (mSupportedExtensions.end() != 
            mSupportedExtensions.find(extension(itr->path()))) {
            mTargetFilesList.insert(itr->path().string());
        }
        
        ++itr;
    }
    
    return 0;
}

const std::string BroadcastUploader::computeMD5(std::string pFilePath)
{
    unsigned char result[MD5_DIGEST_LENGTH] = {0};
    std::string  md5Value;
    int fd = 0;
    struct stat statbuf;
    void *fileBuffer;
    
    
    if((fd = open(pFilePath.c_str(), O_RDONLY)) < 0) {       
        std::cout<<pFilePath.c_str()<<" "<< strerror(errno)<<std::endl;
        exit(-1);
    }

    if(fstat(fd, &statbuf) < 0) {
        std::cout<<"fstat on "<<pFilePath.c_str()<<"failed:  "<< strerror(errno)<<std::endl;
        exit(-1);
    }

    fileBuffer = mmap(0, statbuf.st_size, PROT_READ, MAP_SHARED, fd, 0);

    MD5((unsigned char*) fileBuffer, statbuf.st_size, result);
    munmap(fileBuffer, statbuf.st_size);

    close(fd);
    
    for(int i=0; i<MD5_DIGEST_LENGTH;++i) {
        char buf[2 + 1] = {0};
        sprintf(buf, "%02x", result[i]);
        md5Value += std::string(buf);
    }

    return md5Value;
}

bool BroadcastUploader::confirm()
{
    std::string confirm;
   
    std::cout<<"Found " << mTargetFilesList.size() << " files. Press 'L' to list or 'U' to start the upload.\n";

    std::cin>>confirm;             
    if (confirm == "L") {
        std::copy(mTargetFilesList.begin(),
                  mTargetFilesList.end(),
                  std::ostream_iterator<std::string>(std::cout, "\n"));
        
        std::cin>>confirm;
        if (confirm == "U") {
            return true;
        }
        else {
            std::cout<<"Exit"<<std::endl;
        }
    }
    else if (confirm == "U") {
        return true;
    }
    else {
        std::cout<<"unknown option "<<"'"<<confirm<<"'"<<std::endl;
    }

    return false;
}
